/***************************************************************************
                          loggingfifo.cpp  -  description
                             -------------------
    begin                : Tue May 23 2000
    copyright            : (C) 2000 by Ivan Martinez
    email                : ivan_m_r@email.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "loggingfifo.h"

#include "dualvariable.h"
#include <strstream>
#include "systemvariable.h"

CLoggingFifo::CLoggingFifo()
{
}
CLoggingFifo::~CLoggingFifo()
{
}
/**  */
void CLoggingFifo::Arrange(void* ExtraParameterL = NULL)
{
  strstream ConversionStreamL;

	SetMode(CRTFifo::WRITE);

	ConversionStreamL << Number() << ends;

	SetIdentifier(string("rtf") + ConversionStreamL.str() + "_" +
		Container()->Identifier());
	SetBuffer("&" + Container()->GlobalIdentifier());
	SetBufferSize("sizeof(" + Container()->GlobalIdentifier() + ")");
}
/**  */
string CLoggingFifo::Class() const
{
	return "CLoggingFifo";
}
/**  */
bool CLoggingFifo::DescendantOf(const string& ClassNameL) const
{
	if (ClassNameL == "CLoggingFifo")
		return true;
	else
		return CVariableFifo::DescendantOf(ClassNameL);
}
/**  */
const CDualVariable* CLoggingFifo::Container() const
{
	return (CDualVariable*)CRTFifo::Container();
}
/**  */
void CLoggingFifo::GenerateCode(ostream& StreamL, const CCodeGenerator* CallerPL,
	const string& SectionL, void* ExtraParameterL = NULL) const
{
	if (CallerPL->DescendantOf("CDynamicSystem"))
	{
		if (SectionL == "fifos declaration")
			GenerateDeclaration(StreamL, ExtraParameterL);
		else
			if (SectionL == "fifos initialization")
				GenerateInitialization(StreamL, ExtraParameterL);
			else
				if (SectionL == "fifos destruction")
					GenerateDestruction(StreamL, ExtraParameterL);	
	}
	else
		if (CallerPL->DescendantOf("CLoop"))
			if (SectionL == "variables log")
				GenerateFifoWriting(StreamL, ExtraParameterL);
			else
				if (SectionL == "global variables log")
					GenerateLockedFifoWriting(StreamL, ExtraParameterL);
}
/**  */
void CLoggingFifo::GenerateLockedFifoWriting(ostream& StreamL,
	void* ExtraParameterL = NULL) const
{
	if (Container()->DescendantOf("CSystemVariable"))
	{
		StreamL << "      rtl_spin_lock_irqsave(&" <<
			((CSystemVariable*)Container())->LockIdentifier() << ", flags);" <<
			endl;
		StreamL << "      write(" << Identifier() << ", " << Buffer() << ", " <<
			BufferSize() << ");" << endl;
		StreamL << "      rtl_spin_unlock_irqrestore(&" <<
			((CSystemVariable*)Container())->LockIdentifier() << ", flags);" <<
			endl;
	}
}
/**  */
bool CLoggingFifo::SetNumber(unsigned int uNumber)
{
  strstream Stream;

  Stream << uNumber << ends;
	CVariableFifo::SetNumber(Stream.str());

	if (uNumber >= 0)
		return true;
	else
	{
		AddError(400);
		return false;
	}
}
/**  */
unsigned int CLoggingFifo::Number() const
{
	strstream Stream;
	long long Number;

	Stream << CVariableFifo::Number();
	Stream >> Number;
	return Number;
}
/**  */
bool CLoggingFifo::SetSize(int iSize)
{
  strstream Stream;

  Stream << iSize << ends;
	CVariableFifo::SetSize(Stream.str());

	if (iSize > 0)
		return true;
	else
	{
		AddError(401);
		return false;
	}
}
/**  */
int CLoggingFifo::Size() const
{
	strstream Stream;
	int iSize;

	Stream << CVariableFifo::Size();
	Stream >> iSize;
	return iSize;
}
/**  */
bool CLoggingFifo::AcceptsContainer(const CCodeGenerator* pContainer)
{
	if (pContainer->DescendantOf("CDualVariable"))
	{
		if (pContainer->Components().DescendantsOf("CLoggingFifo"))
		{
			AddError(1900);
			return false;
		}
		else
		{
			SetContainer(pContainer);
			return true;
		}
	}
	else
	{
		AddError(101);
		return false;
	}
}
/**  */
void CLoggingFifo::GenerateCode(CStringsSet& StringsSet,
	const CCodeGenerator* pCaller, const string& Section,
	void* pExtraParameter = NULL) const
{
	if (pCaller->DescendantOf("CDynamicSystem"))
	{
		if (Section == "includes")
			GenerateIncludes(StringsSet, pExtraParameter);
	}
	else
		if (pCaller->DescendantOf("CLoop"))
			if (Section == "local variables declaration")
				GenerateLocalDeclarations(StringsSet, pExtraParameter);
}
/**  */
void CLoggingFifo::GenerateLocalDeclarations(CStringsSet& StringsSet,
	void* pExtraParameter = NULL) const
{
	if (Container()->DescendantOf("CSystemVariable"))
		StringsSet.insert("  static unsigned long flags;");
}
