/***************************************************************************
                          main.cpp  -  description
                             -------------------
    begin                : Tue Jun 13 2000
    copyright            : (C) 2000 by Ivan Martinez
    email                : ivan_m_r@email.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <fstream>
#include <iostream>
#include <stdlib.h>
#include "dynamicsystem.h"
#include "function.h"
#include <vector>
#include "functioninput.h"
#include "functionoutput.h"
#include "systemvariable.h"
#include "loopvariable.h"
#include "loggingfifo.h"
#include "settingfifo.h"
#include "reportingfifo.h"
#include "commandingfifo.h"
#include "functionvariable.h"

int main(int argc, char *argv[])
{
	// Defining a dynamic system
  CDynamicSystem* pDynamicSystem = new CDynamicSystem();
	bool bNoError = true;
	bNoError = pDynamicSystem->SetTime(1000000000);
	bNoError = pDynamicSystem->SetTime(5 * pDynamicSystem->Time());

	// Defining and adding loops
	CLoop* pLoopA = new CLoop();
	bNoError = pLoopA->SetName("LoopA") && bNoError;
	bNoError = pLoopA->SetScheduling(CLoop::PERIOD) && bNoError;
	bNoError = pLoopA->SetPeriod(300000000) && bNoError;
	bNoError = pLoopA->SetPriority(0) && bNoError;
	bNoError = pLoopA->SetFloatingPoint(true) && bNoError;
	CLoop* pLoopB = new CLoop();
	bNoError = pLoopB->SetName("LoopB") && bNoError;
	bNoError = pLoopB->SetScheduling(CLoop::INTERRUPT) && bNoError;
  bNoError = pLoopB->SetIRQNumber(5) && bNoError;
	// bNoError = pLoopB->SetPeriod(130000000) && bNoError;
	// bNoError = pLoopB->SetPriority(2) && bNoError;
	bNoError = pDynamicSystem->AddComponent(pLoopA) && bNoError;
	bNoError = pDynamicSystem->AddComponent(pLoopB) && bNoError;

	// Defining and adding blocks
	CFunction* pFunctionAA = new CFunction();
	bNoError = pFunctionAA->SetName("FunctionAA") && bNoError;
	bNoError = pFunctionAA->SetCode(
		"OutputAA = InputAA + LoopVarA + SysVarBB2AA;") && bNoError;
	CFunction* pFunctionAB = new CFunction();
	bNoError = pFunctionAB->SetName("FunctionAB") && bNoError;
	bNoError = pFunctionAB->SetCode(
    "FuncVarAB = InputAB + LoopVarA;\n"
		"OutputAB = (int)FuncVarAB;\n"
		"SysVarAB2BA = FuncVarAB;") && bNoError;
	CFunction* pFunctionBA = new CFunction();
	bNoError = pFunctionBA->SetName("FunctionBA") && bNoError;
	bNoError = pFunctionBA->SetCode(
		"OutputBA = InputBA + LoopVarB + SysVarAB2BA;") && bNoError;
	CFunction* pFunctionBB = new CFunction();
	bNoError = pFunctionBB->SetName("FunctionBB") && bNoError;
	bNoError = pFunctionBB->SetCode(
    "FuncVarBB = (int)InputBB + (int)LoopVarB;\n"
		"OutputBB = FuncVarBB;\n"
		"SysVarBB2AA = FuncVarBB;") && bNoError;
	bNoError = pLoopA->AddComponent(pFunctionAA) && bNoError;
	bNoError = pLoopA->AddComponent(pFunctionAB) && bNoError;
	bNoError = pLoopB->AddComponent(pFunctionBA) && bNoError;
	bNoError = pLoopB->AddComponent(pFunctionBB) && bNoError;

	// Defining and adding outputs
	CFunctionOutput* pOutputAA = new CFunctionOutput();
	bNoError = pOutputAA->SetName("OutputAA") && bNoError;
	bNoError = pOutputAA->SetType("float") && bNoError;
	CFunctionOutput* pOutputAB = new CFunctionOutput();	
	bNoError = pOutputAB->SetName("OutputAB") && bNoError;
	bNoError = pOutputAB->SetType("int") && bNoError;
	CFunctionOutput* pOutputBA = new CFunctionOutput();
	bNoError = pOutputBA->SetName("OutputBA") && bNoError;
	bNoError = pOutputBA->SetType("float") && bNoError;
	CFunctionOutput* pOutputBB = new CFunctionOutput();
	bNoError = pOutputBB->SetName("OutputBB") && bNoError;
	bNoError = pOutputBB->SetType("int") && bNoError;
  bNoError = pFunctionAA->AddComponent(pOutputAA) && bNoError;
  bNoError = pFunctionAB->AddComponent(pOutputAB) && bNoError;
  bNoError = pFunctionBA->AddComponent(pOutputBA) && bNoError;
  bNoError = pFunctionBB->AddComponent(pOutputBB) && bNoError;

  // Defining and adding inputs
	CFunctionInput* pInputAA = new CFunctionInput();
	bNoError = pInputAA->SetName("InputAA") && bNoError;
	bNoError = pInputAA->SetType("int") && bNoError;
	CFunctionInput* pInputAB = new CFunctionInput();
	bNoError = pInputAB->SetName("InputAB") && bNoError;
	bNoError = pInputAB->SetType("float") && bNoError;
	CFunctionInput* pInputBA = new CFunctionInput();
	bNoError = pInputBA->SetName("InputBA") && bNoError;
	bNoError = pInputBA->SetType("int") && bNoError;
	CFunctionInput* pInputBB = new CFunctionInput();
	bNoError = pInputBB->SetName("InputBB") && bNoError;
	bNoError = pInputBB->SetType("float") && bNoError;
  bNoError = pFunctionAA->AddComponent(pInputAA) && bNoError;
  bNoError = pFunctionAB->AddComponent(pInputAB) && bNoError;
  bNoError = pFunctionBA->AddComponent(pInputBA) && bNoError;
  bNoError = pFunctionBB->AddComponent(pInputBB) && bNoError;

	// Connecting inputs to outputs
	bNoError = pInputAB->Connect(pOutputAA) && bNoError;
	bNoError = pInputAA->Connect(pOutputAB) && bNoError;
  bNoError = pInputBA->Connect(pOutputBB) && bNoError;
  bNoError = pInputBB->Connect(pOutputBA) && bNoError;

	// Defining and adding function variables
	CFunctionVariable* pFuncVarAB = new CFunctionVariable();
	bNoError = pFuncVarAB->SetName("FuncVarAB") && bNoError;
	bNoError = pFuncVarAB->SetType("float") && bNoError;
	CFunctionVariable* pFuncVarBB = new CFunctionVariable();	
	bNoError = pFuncVarBB->SetName("FuncVarBB") && bNoError;
	bNoError = pFuncVarBB->SetType("int") && bNoError;
	bNoError = pFunctionAB->AddComponent(pFuncVarAB) && bNoError;
	bNoError = pFunctionBB->AddComponent(pFuncVarBB) && bNoError;

	// Defining and adding loop variables
	CLoopVariable* pLoopVarA = new CLoopVariable();
	bNoError = pLoopVarA->SetName("LoopVarA") && bNoError;
	bNoError = pLoopVarA->SetType("int") && bNoError;
	CLoopVariable* pLoopVarB = new CLoopVariable();
	bNoError = pLoopVarB->SetName("LoopVarB") && bNoError;
	bNoError = pLoopVarB->SetType("float") && bNoError;
	bNoError = pLoopA->AddComponent(pLoopVarA) && bNoError;
	bNoError = pLoopB->AddComponent(pLoopVarB) && bNoError;

	// Defining and adding system variables
	CSystemVariable* pSysVarAB2BA = new CSystemVariable();
	bNoError = pSysVarAB2BA->SetName("SysVarAB2BA") && bNoError;
	bNoError = pSysVarAB2BA->SetType("float") && bNoError;
	CSystemVariable* pSysVarBB2AA = new CSystemVariable();
	bNoError = pSysVarBB2AA->SetName("SysVarBB2AA") && bNoError;
	bNoError = pSysVarBB2AA->SetType("int") && bNoError;
	bNoError = pDynamicSystem->AddComponent(pSysVarAB2BA) && bNoError;
	bNoError = pDynamicSystem->AddComponent(pSysVarBB2AA) && bNoError;

	// Next step: Arranging the dynamic system
	bNoError = pDynamicSystem->ArrangeAll() && bNoError;

	// Reading the errors report
	CErrors Errors;
	pDynamicSystem->ReportErrors(Errors);
	CErrors::ConstIterator ErrorsIterator;
	for (ErrorsIterator = Errors.Begin(); ErrorsIterator != Errors.End(); ErrorsIterator++)
	{
		cout << (*ErrorsIterator).Number() << endl;
		cout << (*ErrorsIterator).Message() << endl;
	}
  CFunction* pFunction;
	for (ErrorsIterator = Errors.Begin(); ErrorsIterator != Errors.End(); ErrorsIterator++)
    if (((*ErrorsIterator).Location())->Class() == "CFunction")
    {
			pFunction = (CFunction*)((*ErrorsIterator).Location());
			cout << pFunction->Name() << endl;
    }

	// Initializing inputs: Blocks ordenation
	pInputAA->SetInitialization("InputAA = 1;");
	pInputAB->SetInitialization("InputAB = 1;");
  pDynamicSystem->ArrangeAll() && bNoError;
	if (pDynamicSystem->ReportErrors(Errors))
  {
	  CFunctionInput* pFunctionInput;
		for (ErrorsIterator = Errors.Begin(); ErrorsIterator != Errors.End(); ErrorsIterator++)
		{
			cout << (*ErrorsIterator).Number() << endl;
			cout << (*ErrorsIterator).Message() << endl;
	    if (((*ErrorsIterator).Location())->Class() == "CFunctionInput")
	    {
				pFunctionInput = (CFunctionInput*)((*ErrorsIterator).Location());
				cout << pFunctionInput->Name() << endl;
	    }
		}
	}
	bNoError = true;
	bNoError = pInputAB->SetInitialization("") && bNoError;
	bNoError = pInputBB->SetInitialization("InputBB = 1;") && bNoError;

  // Initalizing loop and system variables
	bNoError = pLoopVarA->SetInitialization("LoopVarA = 1;") && bNoError;
	bNoError = pLoopVarB->SetInitialization("LoopVarB = 1;") && bNoError;
	bNoError = pSysVarAB2BA->SetInitialization("SysVarAB2BA = 1;") && bNoError;
	bNoError = pSysVarBB2AA->SetInitialization("SysVarBB2AA = 1;") && bNoError;

	// Last step: Generating the module's code

//  bNoError = pDynamicSystem->ArrangeAll() && bNoError;
	ofstream FileStream;
/*
	FileStream.open("/home/ivan/project/dsl/rtmodule/rtmodule.c");
	pDynamicSystem->GenerateSource(FileStream);
	FileStream.close();
*/

  // Defining, adding and using logging fifos
  CLoggingFifo* pOutputAALogFifo;
  pOutputAALogFifo = new CLoggingFifo();
  bNoError = pOutputAALogFifo->SetNumber(0) && bNoError;
  bNoError = pOutputAALogFifo->SetSize(80) && bNoError;
  bNoError = pOutputAA->AddComponent(pOutputAALogFifo) && bNoError;
  CLoggingFifo* pOutputABLogFifo;
  pOutputABLogFifo = new CLoggingFifo();
  bNoError = pOutputABLogFifo->SetNumber(1) && bNoError;
  bNoError = pOutputABLogFifo->SetSize(110) && bNoError;
  bNoError = pOutputAB->AddComponent(pOutputABLogFifo) && bNoError;
  CLoggingFifo* pOutputBALogFifo;
  pOutputBALogFifo = new CLoggingFifo();
  bNoError = pOutputBALogFifo->SetNumber(2) && bNoError;
  bNoError = pOutputBALogFifo->SetSize(120) && bNoError;
  bNoError = pOutputBA->AddComponent(pOutputBALogFifo) && bNoError;
  CLoggingFifo* pOutputBBLogFifo;
  pOutputBBLogFifo = new CLoggingFifo();
  bNoError = pOutputBBLogFifo->SetNumber(3) && bNoError;
  bNoError = pOutputBBLogFifo->SetSize(130) && bNoError;
  bNoError = pOutputBB->AddComponent(pOutputBBLogFifo) && bNoError;
  CLoggingFifo* pLoopVarALogFifo;
  pLoopVarALogFifo = new CLoggingFifo();
  bNoError = pLoopVarALogFifo->SetNumber(4) && bNoError;
  bNoError = pLoopVarALogFifo->SetSize(140) && bNoError;
  bNoError = pLoopVarA->AddComponent(pLoopVarALogFifo) && bNoError;
  CLoggingFifo* pLoopVarBLogFifo;
  pLoopVarBLogFifo = new CLoggingFifo();
  bNoError = pLoopVarBLogFifo->SetNumber(5) && bNoError;
  bNoError = pLoopVarBLogFifo->SetSize(150) && bNoError;
  bNoError = pLoopVarB->AddComponent(pLoopVarBLogFifo) && bNoError;
  CLoggingFifo* pSysVarAB2BALogFifo;
  pSysVarAB2BALogFifo = new CLoggingFifo();
  bNoError = pSysVarAB2BALogFifo->SetNumber(6) && bNoError;
  bNoError = pSysVarAB2BALogFifo->SetSize(160) && bNoError;
  bNoError = pSysVarAB2BA->AddComponent(pSysVarAB2BALogFifo) && bNoError;
  CLoggingFifo* pSysVarBB2AALogFifo;
  pSysVarBB2AALogFifo = new CLoggingFifo();
  bNoError = pSysVarBB2AALogFifo->SetNumber(7) && bNoError;
  bNoError = pSysVarBB2AALogFifo->SetSize(170) && bNoError;
  bNoError = pSysVarBB2AA->AddComponent(pSysVarBB2AALogFifo) && bNoError;
  bNoError = pDynamicSystem->ArrangeAll() && bNoError;
	FileStream.open("/home/ivan/project/dsl/rtmodule/rtmodule.c");
	pDynamicSystem->GenerateSource(FileStream);
	FileStream.close();

  // Defining, adding and using setting fifos
  CSettingFifo* pInputAASetFifo;
  pInputAASetFifo = new CSettingFifo();
  bNoError = pInputAASetFifo->SetNumber(8) && bNoError;
  bNoError = pInputAASetFifo->SetSize(8) && bNoError;
  bNoError = pInputAA->AddComponent(pInputAASetFifo) && bNoError;
  CSettingFifo* pInputABSetFifo;
  pInputABSetFifo = new CSettingFifo();
  bNoError = pInputABSetFifo->SetNumber(9) && bNoError;
  bNoError = pInputABSetFifo->SetSize(8) && bNoError;
  bNoError = pInputAB->AddComponent(pInputABSetFifo) && bNoError;
  CSettingFifo* pInputBASetFifo;
  pInputBASetFifo = new CSettingFifo();
  bNoError = pInputBASetFifo->SetNumber(10) && bNoError;
  bNoError = pInputBASetFifo->SetSize(8) && bNoError;
  bNoError = pInputBA->AddComponent(pInputBASetFifo) && bNoError;
  CSettingFifo* pInputBBSetFifo;
  pInputBBSetFifo = new CSettingFifo();
  bNoError = pInputBBSetFifo->SetNumber(11) && bNoError;
  bNoError = pInputBBSetFifo->SetSize(8) && bNoError;
  bNoError = pInputBB->AddComponent(pInputBBSetFifo) && bNoError;
  CSettingFifo* pLoopVarASetFifo;
  pLoopVarASetFifo = new CSettingFifo();
  bNoError = pLoopVarASetFifo->SetNumber(12) && bNoError;
  bNoError = pLoopVarASetFifo->SetSize(8) && bNoError;
  bNoError = pLoopVarA->AddComponent(pLoopVarASetFifo) && bNoError;
  CSettingFifo* pLoopVarBSetFifo;
  pLoopVarBSetFifo = new CSettingFifo();
  bNoError = pLoopVarBSetFifo->SetNumber(13) && bNoError;
  bNoError = pLoopVarBSetFifo->SetSize(8) && bNoError;
  bNoError = pLoopVarB->AddComponent(pLoopVarBSetFifo) && bNoError;
  CSettingFifo* pSysVarAB2BASetFifo;
  pSysVarAB2BASetFifo = new CSettingFifo();
  bNoError = pSysVarAB2BASetFifo->SetNumber(14) && bNoError;
  bNoError = pSysVarAB2BASetFifo->SetSize(8) && bNoError;
  bNoError = pSysVarAB2BA->AddComponent(pSysVarAB2BASetFifo) && bNoError;
  CSettingFifo* pSysVarBB2AASetFifo;
  pSysVarBB2AASetFifo = new CSettingFifo();
  bNoError = pSysVarBB2AASetFifo->SetNumber(15) && bNoError;
  bNoError = pSysVarBB2AASetFifo->SetSize(8) && bNoError;
  bNoError = pSysVarBB2AA->AddComponent(pSysVarBB2AASetFifo) && bNoError;

  // Defining, adding and using reporting fifos
  CReportingFifo* pLoopARepFifo;
  pLoopARepFifo = new CReportingFifo();
  bNoError = pLoopARepFifo->SetNumber(16) && bNoError;
  bNoError = pLoopARepFifo->SetSize(320) && bNoError;
  bNoError = pLoopA->AddComponent(pLoopARepFifo) && bNoError;
  CReportingFifo* pLoopBRepFifo;
  pLoopBRepFifo = new CReportingFifo();
  bNoError = pLoopBRepFifo->SetNumber(17) && bNoError;
  bNoError = pLoopBRepFifo->SetSize(320) && bNoError;
  bNoError = pLoopB->AddComponent(pLoopBRepFifo) && bNoError;
  bNoError = pDynamicSystem->ArrangeAll() && bNoError;
	FileStream.open("/home/ivan/project/dsl/rtmodule/rtmodule.c");
	pDynamicSystem->GenerateSource(FileStream);
	FileStream.close();

	// Defining, adding and using a commanding fifo
  CCommandingFifo* pCommandingFifo;
	pCommandingFifo = new CCommandingFifo();
	bNoError = pCommandingFifo->SetNumber(18) && bNoError;
  bNoError = pCommandingFifo->SetSize(100) && bNoError;
	bNoError = pDynamicSystem->AddComponent(pCommandingFifo) && bNoError;
  pDynamicSystem->ArrangeAll();
	FileStream.open("/home/ivan/project/dsl/rtmodule/rtmodule.c");
	pDynamicSystem->GenerateSource(FileStream);
	FileStream.close();

	pDynamicSystem->DeleteObjects();
  delete pDynamicSystem;

  return EXIT_SUCCESS;
}
